@@ -13,8 +13,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <algorithm>  // std::fill
-#include <cmath>      // std::isfinite
+#include <stddef.h>
+#include <stdint.h>
 
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "tests/demote_test.cc"
@@ -28,16 +28,7 @@
 HWY_BEFORE_NAMESPACE();
 namespace hwy {
 namespace HWY_NAMESPACE {
-
-template <typename T, HWY_IF_FLOAT(T)>
-bool IsFiniteT(T t) {
-  return std::isfinite(t);
-}
-// Wrapper avoids calling std::isfinite for integer types (ambiguous).
-template <typename T, HWY_IF_NOT_FLOAT(T)>
-bool IsFiniteT(T /*unused*/) {
-  return true;
-}
+namespace {
 
 template <typename ToT>
 struct TestDemoteTo {
@@ -53,14 +44,14 @@ struct TestDemoteTo {
     HWY_ASSERT(from && expected);
 
     // Narrower range in the wider type, for clamping before we cast
-    const T min = static_cast<T>(IsSigned<T>() ? LimitsMin<ToT>() : ToT{0});
+    const T min = ConvertScalarTo<T>(IsSigned<T>() ? LimitsMin<ToT>()
+                                                   : static_cast<ToT>(0));
     const T max = LimitsMax<ToT>();
 
     RandomState rng;
     for (size_t rep = 0; rep < AdjustedReps(1000); ++rep) {
       for (size_t i = 0; i < N; ++i) {
-        const uint64_t bits = rng();
-        CopyBytes<sizeof(T)>(&bits, &from[i]);  // not same size
+        from[i] = RandomFiniteValue<T>(&rng);
         expected[i] = static_cast<ToT>(HWY_MIN(HWY_MAX(min, from[i]), max));
       }
       const auto in = Load(from_d, from.get());
@@ -76,7 +67,7 @@ struct TestDemoteTo {
           expected[i] &= static_cast<ToT>(max);
         }
 
-        from[i] = static_cast<T>(expected[i]);
+        from[i] = ConvertScalarTo<T>(expected[i]);
       }
 
       const auto in = Load(from_d, from.get());
@@ -158,24 +149,128 @@ struct TestDemoteToFloat {
     static_assert(sizeof(T) > sizeof(ToT), "Input type must be wider");
     const Rebind<ToT, D> to_d;
 
+    using TU = MakeUnsigned<T>;
+    using ToTU = MakeUnsigned<ToT>;
+
     const size_t N = Lanes(from_d);
     auto from = AllocateAligned<T>(N);
     auto expected = AllocateAligned<ToT>(N);
     HWY_ASSERT(from && expected);
 
+    constexpr int kMaxToBiasedExp = static_cast<int>(MaxExponentField<ToT>());
+    static_assert(kMaxToBiasedExp > 0, "kMaxToBiasedExp > 0 must be true");
+
+    constexpr int kMaxFromBiasedExp = static_cast<int>(MaxExponentField<T>());
+    static_assert(kMaxFromBiasedExp >= kMaxToBiasedExp,
+                  "kMaxFromBiasedExp >= kMaxToBiasedExp must be true");
+
+    constexpr int kMaxFromExpBias = (kMaxFromBiasedExp / 2);
+    constexpr int kMaxToExpBias = (kMaxToBiasedExp / 2);
+
+    constexpr int kMinToTNormalFromTBiasedExp =
+        1 - kMaxToExpBias + kMaxFromExpBias;
+    static_assert(kMinToTNormalFromTBiasedExp > 0,
+                  "kMinToTNormalFromTBiasedExp must be greater than 0");
+    static_assert(
+        kMinToTNormalFromTBiasedExp < kMaxFromExpBias,
+        "kMinToTNormalFromTBiasedExp must be less than kMaxFromExpBias");
+
+    const T min_normal = BitCastScalar<T>(static_cast<TU>(
+        static_cast<TU>(kMinToTNormalFromTBiasedExp) << MantissaBits<T>()));
+    HWY_ASSERT(ScalarIsFinite(min_normal));
+    HWY_ASSERT(min_normal > static_cast<T>(0) &&
+               min_normal < static_cast<T>(1));
+
+    const T max_denormal =
+        ConvertScalarTo<T>(BitCastScalar<ToT>(MantissaMask<ToT>()));
+    HWY_ASSERT(ScalarIsFinite(max_denormal));
+    HWY_ASSERT(max_denormal > static_cast<T>(0) && max_denormal < min_normal);
+
+    const T min_denormal = ConvertScalarTo<T>(BitCastScalar<ToT>(ToTU{1}));
+    HWY_ASSERT(ScalarIsFinite(min_denormal));
+    HWY_ASSERT(min_denormal > static_cast<T>(0) && min_denormal < max_denormal);
+
+    const T half_min_denormal =
+        static_cast<T>(min_denormal * static_cast<T>(0.5));
+    HWY_ASSERT(ScalarIsFinite(half_min_denormal));
+    HWY_ASSERT(half_min_denormal > static_cast<T>(0) &&
+               half_min_denormal < min_denormal);
+
+    const T max_abs = ConvertScalarTo<T>(HighestValue<ToT>());
+    HWY_ASSERT(ScalarIsFinite(max_abs));
+    HWY_ASSERT(max_abs > static_cast<T>(1));
+
+    const T min_out_of_range = BitCastScalar<T>(
+        static_cast<TU>((BitCastScalar<TU>(max_abs) & ExponentMask<T>()) +
+                        (TU{1} << MantissaBits<T>())));
+    HWY_ASSERT(!ScalarIsNaN(min_out_of_range));
+    HWY_ASSERT(max_abs < min_out_of_range);
+
+    const ToTU kToTPosInfBits = ExponentMask<ToT>();
+
     RandomState rng;
+
+    // Check that values that are within the range of a normal finite ToT are
+    // converted to a correctly rounded normal value
     for (size_t rep = 0; rep < AdjustedReps(1000); ++rep) {
       for (size_t i = 0; i < N; ++i) {
-        do {
-          const uint64_t bits = rng();
-          CopyBytes<sizeof(T)>(&bits, &from[i]);  // not same size
-        } while (!IsFiniteT(from[i]));
-        const T magn = std::abs(from[i]);
-        const T max_abs = HighestValue<ToT>();
-        // NOTE: std:: version from C++11 cmath is not defined in RVV GCC, see
-        // https://lists.freebsd.org/pipermail/freebsd-current/2014-January/048130.html
-        const T clipped = copysign(HWY_MIN(magn, max_abs), from[i]);
-        expected[i] = static_cast<ToT>(clipped);
+        const T rand_val = RandomFiniteValue<T>(&rng);
+        const T magn = ScalarAbs(rand_val);
+        const T clipped = ScalarCopySign(
+            HWY_MIN(HWY_MAX(magn, min_normal), max_abs), rand_val);
+        from[i] = clipped;
+        expected[i] = ConvertScalarTo<ToT>(clipped);
+      }
+
+      HWY_ASSERT_VEC_EQ(to_d, expected.get(),
+                        DemoteTo(to_d, Load(from_d, from.get())));
+    }
+
+    // Check that values that are between min_denormal and max_denormal are
+    // converted to a correctly rounded denormal value
+    for (size_t rep = 0; rep < AdjustedReps(1000); ++rep) {
+      for (size_t i = 0; i < N; ++i) {
+        const T rand_val = RandomFiniteValue<T>(&rng);
+        const T magn = ScalarAbs(rand_val);
+        const T clipped = ScalarCopySign(
+            HWY_MIN(HWY_MAX(magn, min_denormal), max_denormal), rand_val);
+        from[i] = clipped;
+        expected[i] = ConvertScalarTo<ToT>(clipped);
+      }
+
+      HWY_ASSERT_VEC_EQ(to_d, expected.get(),
+                        DemoteTo(to_d, Load(from_d, from.get())));
+    }
+
+    // Check that denormal values whose absolute value is less than or equal to
+    // half_min_denormal is converted to zero
+    for (size_t rep = 0; rep < AdjustedReps(1000); ++rep) {
+      for (size_t i = 0; i < N; ++i) {
+        const T rand_val = RandomFiniteValue<T>(&rng);
+        const T magn = ScalarAbs(rand_val);
+        const T clipped =
+            ScalarCopySign(HWY_MIN(magn, half_min_denormal), rand_val);
+        from[i] = clipped;
+      }
+
+      HWY_ASSERT_VEC_EQ(to_d, Zero(to_d),
+                        DemoteTo(to_d, Load(from_d, from.get())));
+    }
+
+    // Check that finite values that are out of the range of ToT are correctly
+    // converted to positive infinity or negative infinity
+    for (size_t rep = 0; rep < AdjustedReps(1000); ++rep) {
+      for (size_t i = 0; i < N; ++i) {
+        const T rand_val = RandomFiniteValue<T>(&rng);
+        const T abs_rand_val = ScalarAbs(rand_val);
+        const T rand_out_of_range_val =
+            ScalarCopySign(HWY_MAX(abs_rand_val, min_out_of_range), rand_val);
+
+        from[i] = rand_out_of_range_val;
+
+        expected[i] = BitCastScalar<ToT>(static_cast<ToTU>(
+            kToTPosInfBits | (static_cast<ToTU>(ScalarSignBit(rand_val))
+                              << (sizeof(ToTU) * 8 - 1))));
       }
 
       HWY_ASSERT_VEC_EQ(to_d, expected.get(),
@@ -185,12 +280,16 @@ struct TestDemoteToFloat {
 };
 
 HWY_NOINLINE void TestAllDemoteToFloat() {
-  // Must test f16 separately because we can only load/store/convert them.
-
 #if HWY_HAVE_FLOAT64
   const ForDemoteVectors<TestDemoteToFloat<float>, 1> to_float;
   to_float(double());
+
+  const ForDemoteVectors<TestDemoteToFloat<hwy::float16_t>, 2> f64_to_f16;
+  f64_to_f16(double());
 #endif
+
+  const ForDemoteVectors<TestDemoteToFloat<hwy::float16_t>, 1> f32_to_f16;
+  f32_to_f16(float());
 }
 
 struct TestDemoteUI64ToFloat {
@@ -207,18 +306,18 @@ struct TestDemoteUI64ToFloat {
   HWY_NOINLINE void operator()(T /*unused*/, D from_d) {
     const Rebind<float, D> df32;
 
-    Verify(from_d, T{0}, 0.0f);
+    Verify(from_d, static_cast<T>(0), 0.0f);
     Verify(from_d, LimitsMax<T>(), static_cast<float>(LimitsMax<T>()));
-    Verify(from_d, T(11808), 11808.0f);
-    Verify(from_d, T(261162016), 261162016.0f);
-    Verify(from_d, T(18665497952256LL), 18665497952256.0f);
+    Verify(from_d, static_cast<T>(11808), 11808.0f);
+    Verify(from_d, static_cast<T>(261162016), 261162016.0f);
+    Verify(from_d, static_cast<T>(18665497952256LL), 18665497952256.0f);
 
     if (IsSigned<T>()) {
-      Verify(from_d, T(-1), -1.0f);
+      Verify(from_d, static_cast<T>(-1), -1.0f);
       Verify(from_d, LimitsMin<T>(), static_cast<float>(LimitsMin<T>()));
-      Verify(from_d, T(-17633), -17633.0f);
-      Verify(from_d, T(-3888877568LL), -3888877568.0f);
-      Verify(from_d, T(-17851503083520LL), -17851503083520.0f);
+      Verify(from_d, static_cast<T>(-17633), -17633.0f);
+      Verify(from_d, static_cast<T>(-3888877568LL), -3888877568.0f);
+      Verify(from_d, static_cast<T>(-17851503083520LL), -17851503083520.0f);
     }
 
     const size_t N = Lanes(from_d);
@@ -269,10 +368,7 @@ struct TestDemoteToBF16 {
     RandomState rng;
     for (size_t rep = 0; rep < AdjustedReps(1000); ++rep) {
       for (size_t i = 0; i < N; ++i) {
-        do {
-          const uint64_t bits = rng();
-          CopyBytes<sizeof(T)>(&bits, &from[i]);  // not same size
-        } while (!IsFiniteT(from[i]));
+        from[i] = RandomFiniteValue<T>(&rng);
 
         uint32_t fromBits;
         CopyBytes<sizeof(uint32_t)>(&from[i], &fromBits);
@@ -363,8 +459,8 @@ AlignedFreeUniquePtr<float[]> ReorderBF16TestCases(D d, size_t& padded) {
   auto in = AllocateAligned<float>(padded);
   auto expected = AllocateAligned<float>(padded);
   HWY_ASSERT(in && expected);
-  std::copy(test_cases, test_cases + kNumTestCases, in.get());
-  std::fill(in.get() + kNumTestCases, in.get() + padded, 0.0f);
+  CopyBytes(test_cases, in.get(), kNumTestCases * sizeof(float));
+  ZeroBytes(in.get() + kNumTestCases, (padded - kNumTestCases) * sizeof(float));
   return in;
 }
 
@@ -469,7 +565,7 @@ class TestIntegerReorderDemote2To {
     HWY_ASSERT(from && expected && actual);
 
     // Narrower range in the wider type, for clamping before we cast
-    const T min = static_cast<T>(IsSigned<T>() ? LimitsMin<TN>() : TN{0});
+    const T min = ConvertScalarTo<T>(IsSigned<T>() ? LimitsMin<TN>() : TN{0});
     const T max = LimitsMax<TN>();
 
     RandomState rng;
@@ -497,7 +593,7 @@ class TestIntegerReorderDemote2To {
           expected[i] &= static_cast<TN>(max);
         }
 
-        from[i] = static_cast<T>(expected[i]);
+        from[i] = ConvertScalarTo<T>(expected[i]);
       }
 
       const auto in_1 = Load(d, from.get());
@@ -533,17 +629,18 @@ HWY_NOINLINE void TestAllReorderDemote2To() {
 }
 
 struct TestFloatOrderedDemote2To {
-  template <typename TF, class DF>
-  HWY_NOINLINE void operator()(TF /*t*/, DF df) {
+  template <typename TN, class DN>
+  HWY_NOINLINE void operator()(TN /*t*/, DN dn) {
 #if HWY_TARGET != HWY_SCALAR
-    const Repartition<bfloat16_t, decltype(df)> dbf16;
-    const RebindToUnsigned<decltype(dbf16)> du16;
+    const RepartitionToWide<decltype(dn)> df;
+    using TF = TFromD<decltype(df)>;
+    const RebindToUnsigned<decltype(dn)> du16;
     const RebindToUnsigned<decltype(df)> du32;
     const Half<decltype(du16)> du16_half;
     const size_t N = Lanes(df);
     const size_t twiceN = N * 2;
     auto from = AllocateAligned<TF>(twiceN);
-    auto expected = AllocateAligned<bfloat16_t>(twiceN);
+    auto expected = AllocateAligned<TN>(twiceN);
     HWY_ASSERT(from && expected);
 
     const auto u16_zero_vect = Zero(du16);
@@ -552,10 +649,7 @@ struct TestFloatOrderedDemote2To {
     RandomState rng;
     for (size_t rep = 0; rep < AdjustedReps(1000); ++rep) {
       for (size_t i = 0; i < twiceN; ++i) {
-        do {
-          const uint64_t bits = rng();
-          CopyBytes<sizeof(TF)>(&bits, &from[i]);  // not same size
-        } while (!IsFiniteT(from[i]));
+        from[i] = RandomFiniteValue<TF>(&rng);
 
         uint32_t u32Bits;
         CopyBytes<sizeof(uint32_t)>(&from[i], &u32Bits);
@@ -563,16 +657,16 @@ struct TestFloatOrderedDemote2To {
         const uint16_t expected_bf16_bits =
             static_cast<uint16_t>(u32Bits >> 16);
 
-        CopyBytes<sizeof(bfloat16_t)>(&expected_bf16_bits, &expected[i]);
+        CopyBytes<sizeof(TN)>(&expected_bf16_bits, &expected[i]);
       }
 
       const auto in_1 = Load(df, from.get());
       const auto in_2 = Load(df, from.get() + N);
-      const auto actual = OrderedDemote2To(dbf16, in_1, in_2);
+      const auto actual = OrderedDemote2To(dn, in_1, in_2);
 
       // Adjust expected to account for any possible rounding that was
       // carried out by the OrderedDemote2To operation
-      auto expected_vect = BitCast(du16, Load(dbf16, expected.get()));
+      auto expected_vect = BitCast(du16, Load(dn, expected.get()));
 
       const auto low_f32_bits =
           Combine(du16, TruncateTo(du16_half, BitCast(du32, in_2)),
@@ -593,12 +687,11 @@ struct TestFloatOrderedDemote2To {
       expected_vect = Add(expected_vect, expected_adj);
 
       // Store the adjusted expected_vect back into expected
-      Store(BitCast(dbf16, expected_vect), dbf16, expected.get());
-
-      HWY_ASSERT_VEC_EQ(dbf16, expected.get(), actual);
+      Store(BitCast(dn, expected_vect), dn, expected.get());
+      HWY_ASSERT_VEC_EQ(dn, expected.get(), actual);
     }
 #else
-    (void)df;
+    (void)dn;
 #endif
   }
 };
@@ -618,7 +711,7 @@ class TestIntegerOrderedDemote2To {
     HWY_ASSERT(from && expected);
 
     // Narrower range in the wider type, for clamping before we cast
-    const T min = static_cast<T>(IsSigned<T>() ? LimitsMin<TN>() : TN{0});
+    const T min = ConvertScalarTo<T>(IsSigned<T>() ? LimitsMin<TN>() : TN{0});
     const T max = LimitsMax<TN>();
 
     RandomState rng;
@@ -643,7 +736,7 @@ class TestIntegerOrderedDemote2To {
           expected[i] &= static_cast<TN>(max);
         }
 
-        from[i] = static_cast<T>(expected[i]);
+        from[i] = ConvertScalarTo<T>(expected[i]);
       }
 
       const auto in_1 = Load(d, from.get());
@@ -672,7 +765,9 @@ class TestIntegerOrderedDemote2To {
 
 HWY_NOINLINE void TestAllOrderedDemote2To() {
   ForUI163264(ForShrinkableVectors<TestIntegerOrderedDemote2To>());
-  ForShrinkableVectors<TestFloatOrderedDemote2To>()(float());
+  ForShrinkableVectors<TestFloatOrderedDemote2To>()(bfloat16_t());
+  // TODO(janwas): replace previous line with this once supported
+  // ForSpecialTypes(ForShrinkableVectors<TestFloatOrderedDemote2To>());
 }
 
 struct TestI32F64 {
@@ -683,25 +778,28 @@ struct TestI32F64 {
     const size_t N = Lanes(df);
 
     // Integer positive
-    HWY_ASSERT_VEC_EQ(di, Iota(di, TI(4)), DemoteTo(di, Iota(df, TF(4.0))));
+    HWY_ASSERT_VEC_EQ(di, Iota(di, 4), DemoteTo(di, Iota(df, 4.0)));
 
     // Integer negative
-    HWY_ASSERT_VEC_EQ(di, Iota(di, -TI(N)), DemoteTo(di, Iota(df, -TF(N))));
+    HWY_ASSERT_VEC_EQ(di, Iota(di, -static_cast<TI>(N)),
+                      DemoteTo(di, Iota(df, -ConvertScalarTo<TF>(N))));
 
     // Above positive
-    HWY_ASSERT_VEC_EQ(di, Iota(di, TI(2)), DemoteTo(di, Iota(df, TF(2.001))));
+    HWY_ASSERT_VEC_EQ(di, Iota(di, 2), DemoteTo(di, Iota(df, 2.001)));
 
     // Below positive
-    HWY_ASSERT_VEC_EQ(di, Iota(di, TI(3)), DemoteTo(di, Iota(df, TF(3.9999))));
+    HWY_ASSERT_VEC_EQ(di, Iota(di, 3), DemoteTo(di, Iota(df, 3.9999)));
 
     const TF eps = static_cast<TF>(0.0001);
     // Above negative
-    HWY_ASSERT_VEC_EQ(di, Iota(di, -TI(N)),
-                      DemoteTo(di, Iota(df, -TF(N + 1) + eps)));
+    HWY_ASSERT_VEC_EQ(
+        di, Iota(di, -static_cast<TI>(N)),
+        DemoteTo(di, Iota(df, -ConvertScalarTo<TF>(N + 1) + eps)));
 
     // Below negative
-    HWY_ASSERT_VEC_EQ(di, Iota(di, -TI(N + 1)),
-                      DemoteTo(di, Iota(df, -TF(N + 1) - eps)));
+    HWY_ASSERT_VEC_EQ(
+        di, Iota(di, -static_cast<TI>(N + 1)),
+        DemoteTo(di, Iota(df, -ConvertScalarTo<TF>(N + 1) - eps)));
 
     // Huge positive float
     HWY_ASSERT_VEC_EQ(di, Set(di, LimitsMax<TI>()),
@@ -719,6 +817,7 @@ HWY_NOINLINE void TestAllI32F64() {
 #endif
 }
 
+}  // namespace
 // NOLINTNEXTLINE(google-readability-namespace-comments)
 }  // namespace HWY_NAMESPACE
 }  // namespace hwy
@@ -727,8 +826,8 @@ HWY_AFTER_NAMESPACE();
 #endif  //  !HWY_IS_MSAN
 
 #if HWY_ONCE
-
 namespace hwy {
+namespace {
 #if !HWY_IS_MSAN
 HWY_BEFORE_TEST(HwyDemoteTest);
 HWY_EXPORT_AND_TEST_P(HwyDemoteTest, TestAllDemoteToInt);
@@ -739,7 +838,9 @@ HWY_EXPORT_AND_TEST_P(HwyDemoteTest, TestAllDemoteToBF16);
 HWY_EXPORT_AND_TEST_P(HwyDemoteTest, TestAllReorderDemote2To);
 HWY_EXPORT_AND_TEST_P(HwyDemoteTest, TestAllOrderedDemote2To);
 HWY_EXPORT_AND_TEST_P(HwyDemoteTest, TestAllI32F64);
+HWY_AFTER_TEST();
 #endif  //  !HWY_IS_MSAN
+}  // namespace
 }  // namespace hwy
-
-#endif
+HWY_TEST_MAIN();
+#endif  // HWY_ONCE
